/*===========================================//
 * acha.cpp
 * ACHA 프로그램의 메인 클래스를 구현한다.
 * 모든 클래스들을 가지고 메인 루틴을 가진다.
 * Written by JH. Cho.
 *===========================================*/
#include "acha.h"
#include "cm.h"
#include "sm.h"
#include "lm.h"
#include "fm.h"
#include "uim.h"

//--------------------------------------------------------------
// 메인함수 : ACHA 프로그램의 메인
//--------------------------------------------------------------
int main(int argc, char **argv)
{
	ACHA acha;

	acha.init();
#ifdef ACHATEST
	acha.debugON();
#endif

	// Demon Run
	if( 2 == argc && 0 == strcmp(argv[1], "-d") )
	{
		acha.m_bDemonMode = true;
		if( fork() != 0 )
			exit(0);
		else
		{
			close(1);
			close(2);
			setsid();
			umask(0);
		}
	}

	acha.run();

	acha.cleanup();

	return 0;
}

ACHA::ACHA(void)
{
	bzero( recordLockPath, sizeof(recordLockPath) );
	bzero( username, sizeof(username) );
	bzero( userpass, sizeof(userpass) );
}

ACHA::~ACHA(void)
{
}

//--------------------------------------------------------------
// 함수 : 초기화
// 설명 : 프로그램 중복실행 방지
// 리턴 : 성공 true 실패 false
//--------------------------------------------------------------
int ACHA::init(void)
{
	// LogManager 설정
	lm.SetOutputMode(OUTPUT_FILE);
	lm.SetLogLevel(LV_INFO);
	lm.SetOwner("ACHA");

#ifdef TEST
	lm.SetLogLevel(LV_TRACE);
#endif

	// 프로그램 중복 실행 방지 루틴
	m_fdRecordLock = -1;
	bzero( recordLockPath, sizeof(recordLockPath) );
	strncpy( recordLockPath, CM_RLOCK_PATH, sizeof(recordLockPath) );
	strncat( recordLockPath, username, sizeof(recordLockPath) - strlen(username) - 1);

	// 프로그램 중복실행 방지
	if( !recordLock() )
	{
		lm.log(LV_ERROR, "프로그램이 이미 실행중입니다.");
		return RC_ERROR;
	}

	// 로그인 ID를 설정한다.
	bzero( username, sizeof(username) );
	strncpy( username, getlogin(), sizeof(username) );
#ifndef TEST
	setPassword();
#endif

	// 기타 설정 
	m_bDemonMode 	= false;
	m_bLock			= false;
	m_bDebug		= false;

	// 각 멤버 클래스를 생성
	m_pCommunicationManager = NULL;
	m_pSyncManager			= NULL;
	m_pFileManager			= NULL;
	m_pUIModule				= NULL;

	m_pCommunicationManager = new CommunicationManager(this);
	m_pSyncManager			= new SyncManager(this);
	m_pFileManager			= new FileManager(this);
	m_pUIModule				= new UIModule(this);

	CommunicationManager 	*cm	= (CommunicationManager *)m_pCommunicationManager;
	SyncManager				*sm	= (SyncManager *)m_pSyncManager;
	FileManager				*fm = (FileManager *)m_pFileManager;
	UIModule				*ui = (UIModule *)m_pUIModule;

	cm->init();
	sm->init();
	fm->init();
	ui->init();

	return RC_OK;
}

int ACHA::run(void)
{
	CommunicationManager 	*cm	= (CommunicationManager *)m_pCommunicationManager;
	SyncManager				*sm	= (SyncManager *)m_pSyncManager;
	FileManager				*fm = (FileManager *)m_pFileManager;
	UIModule				*ui = (UIModule *)m_pUIModule;

	cm->run();
	sm->run();
	fm->run();
	ui->run();

	if( !m_bDemonMode )
	{
		while( getCmd() )
		{
			cmdParsing();
		}
	}
	else
	{
		pause();
	}

	return RC_OK;
}

int ACHA::cleanup(void)
{
	CommunicationManager 	*cm	= (CommunicationManager *)m_pCommunicationManager;
	SyncManager				*sm	= (SyncManager *)m_pSyncManager;
	FileManager				*fm = (FileManager *)m_pFileManager;
	UIModule				*ui = (UIModule *)m_pUIModule;

	cm->cleanup();
	sm->cleanup();
	fm->cleanup();
	ui->cleanup();

	delete cm;
	delete sm;
	delete fm;
	delete ui;

	recordUnlock();

	return RC_OK;
}

/*
 * 함수 : 사용자로부터 명령을 입력 받음
 * 설명 : UI와 network, 그외 다른 클래스로부터 명령을 입력 받음
 *        디버깅을 위한 함수
 */
bool ACHA::getCmd( void )
{
	if(m_bDemonMode) pause();

	bzero( m_cmdBuffer, sizeof(m_cmdBuffer) );

	setcolor( COLOR_GREEN );
	cout << "ACHA::getCmd() > ";
	setcolor( COLOR_NONE );

	fgets( m_cmdBuffer, DATASIZ-1, stdin );
	m_cmdBuffer[strlen(m_cmdBuffer)-1] = '\0';		// m_cmdBuffer 에서 '\n' 제거
	lm.log(LV_TRACE, "%s", m_cmdBuffer);

	if( strcmp( m_cmdBuffer, "q" ) == 0 ||
	 	strncmp( m_cmdBuffer, "exit", strlen("exit") ) == 0 )
	{
		return false;
	}
	
	return true;
}

/*
 * 함수 : 사용자로부터 명령을 입력 받음
 * 설명 : UI와 network, 그외 다른 클래스로부터 명령을 입력 받음
 *        디버깅을 위한 함수
 */
int ACHA::getCmd( char* cmd )
{
	bzero( m_cmdBuffer, sizeof(m_cmdBuffer) );
	strncpy( m_cmdBuffer, cmd, sizeof(m_cmdBuffer) );
	
	return cmdParsing();
}

/*
 * 함수 : 받은 데이터 파싱
 * 설명 : 받은 데이터에 대해 해당 함수를 호출한다.
 *        전송 받은 데이터는 멤버변수에 저장된다.
 * 반환 : SendCmd 형 플래그
 */
int ACHA::cmdParsing( void )
{
	CommunicationManager 	*cm	= (CommunicationManager *)m_pCommunicationManager;
	SyncManager				*sm	= (SyncManager *)m_pSyncManager;
//	FileManager				*fm = (FileManager *)m_pFileManager;
//	UIModule				*ui = (UIModule *)m_pUIModule;

	int cmdLength = strlen( m_cmdBuffer );

	// 엔터 무시
	if( cmdLength == 0 )
	{
		return true;
	}
	
	//---------------------------------------------------------------//
	// 프로그램 사용 기능들
	//---------------------------------------------------------------//
	// 동기화를 생성한다.
	// usage : add syncname targetAddr localpath remotepath
	if( strncmp( m_cmdBuffer, "add ", strlen("add ") ) == 0  && 
		argc( m_cmdBuffer ) == 5 || argc( m_cmdBuffer ) == 6 )
	{
		char *syncname, *target, *lp, *rp;
		strtok( m_cmdBuffer, " " );
		syncname = strtok( NULL, " " );
		target 	 = strtok( NULL, " " );
		lp 		 = strtok( NULL, " " );
		rp 		 = strtok( NULL, " \0" );
		if( sm->addSync( syncname, target, lp, rp, FLAG_NEWSYNC, 
						 "id", "pass", DEFAULT_SYNC_FILE_SIZE) )
			return CM_ACK_OK;
		else
			return CM_ACK_NO;
	}

	// 동기화를 제거한다.
	// usage : del syncname
	if( strncmp( m_cmdBuffer, "del ", strlen("del ") ) == 0  && 
		argc( m_cmdBuffer ) == 2 || argc( m_cmdBuffer ) == 3 )
	{
		if( sm->delSync( &m_cmdBuffer[4] ) )
			return CM_ACK_OK;
		else
			return CM_ACK_NO;
	}

	// 새로운 디렉토리를 만든다. (로컬, 원격지 모두)
	// usage : adddir testdir
	if( strncmp( m_cmdBuffer, "adddir ", strlen("adddir ") ) == 0  && 
		argc( m_cmdBuffer ) == 2 )
	{
		cm->clientData.type = CM_ADD_DIR;
		bzero( cm->clientData.data, sizeof(cm->clientData.data) );

		char filename[DATASIZ] = "";
		struct stat status;
		mode_t filePermission;

		sscanf( &m_cmdBuffer[7], "%s", filename );
		stat( filename, &status );
		filePermission 	= status.st_mode;
		sprintf( cm->clientData.data, "%s %d", filename, filePermission );
		cm->clientData.len  = strlen( cm->clientData.data );
		return cm->sendCmd();
	}

	// 디렉토리를 삭제한다. (로컬, 원격지 모두)
	// usage : deldir testdir
	if( strncmp( m_cmdBuffer, "deldir ", strlen("deldir ") ) == 0  && 
		argc( m_cmdBuffer ) == 2 )
	{
		cm->clientData.type = CM_DEL_DIR;
		bzero( cm->clientData.data, sizeof(cm->clientData.data) );

		char filename[DATASIZ] = "";
		sscanf( &m_cmdBuffer[7], "%s", filename );
		sprintf( cm->clientData.data, "%s", filename );
		cm->clientData.len  = strlen( cm->clientData.data );
		return cm->sendCmd();
	}

	// 파일을 삭제한다. (로컬, 원격지 모두)
	// usage : delfile filename
	if( strncmp( m_cmdBuffer, "delfile ", strlen("delfile ") ) == 0  && 
		argc( m_cmdBuffer ) == 2 )
	{
		cm->clientData.type = CM_DEL_FILE;
		bzero( cm->clientData.data, sizeof(cm->clientData.data) );

		char filename[DATASIZ] = "";
		sscanf( &m_cmdBuffer[8], "%s", filename );
		sprintf( cm->clientData.data, "%s", filename );
		cm->clientData.len  = strlen( cm->clientData.data );
		return cm->sendCmd();
	}

	// 파일 생성한다. (로컬, 원격지 모두)
	// usage : addfile filename
	if( strncmp( m_cmdBuffer, "addfile ", strlen("addfile ") ) == 0  && 
		argc( m_cmdBuffer ) == 3 )
	{
		cm->clientData.type = CM_ADD_FILE;
		bzero( cm->clientData.data, sizeof(cm->clientData.data) );

		char filename[DATASIZ] = "";
		bzero( m_md5buf, sizeof(m_md5buf) );
		sscanf( &m_cmdBuffer[8], "%s %s", filename, m_md5buf);
		sprintf( cm->clientData.data, "%s %s", filename, m_md5buf );
		cm->clientData.len  = strlen( cm->clientData.data );

		switch( cm->sendCmd() )
		{
			case CM_ACK_OK :
				// 파일을 전송해야 할 부분
				return CM_ACK_OK;
			case CM_ACK_HASH_EQUAL :
				// 파일의 해쉬코드가 같음 전송 필요 없음
				return CM_ACK_HASH_EQUAL;
			default :
				// 정책상 밀리는 파일일 경우
				return CM_ACK_NO;
		}
	}

	// 최종 동기화 시간을 현재시간으로 갱신한다.
	if( strncmp( m_cmdBuffer, "update ", strlen("update ") ) == 0  && 
		argc( m_cmdBuffer ) == 2 )
	{
		sm->updateSyncTime( &m_cmdBuffer[7], time(NULL) );
		return CM_ACK_OK;
	}

	// 원격지로 접속 할 포트를 설정한다.
	// usage : port 8080
	if( strncmp( m_cmdBuffer, "port ", strlen("port ") ) == 0  && 
		argc( m_cmdBuffer ) == 2 )
	{
		sscanf( &m_cmdBuffer[5], "%d", &cm->sendPort );
	}

	// 원격지로 접속 할 타겟을 설정한다.
	// usage : target open.ssu.ac.kr
	if( strncmp( m_cmdBuffer, "target ", strlen("target ") ) == 0  && 
		argc( m_cmdBuffer ) == 2 )
	{
		bzero( cm->remoteTarget, sizeof(cm->remoteTarget) );
		sscanf( &m_cmdBuffer[7], "%s", cm->remoteTarget );
		return CM_ACK_OK;
	}

	// 원격지로 접속한다.
	// 현재 Target, Port 로 지정되어 있는 곳으로 접속
	if( strcmp( m_cmdBuffer, "connect" ) == 0 )
	{
		if( m_bDebug )
		{
			setcolor( COLOR_YELLOW );
			cout << "CM::connect : " << cm->remoteTarget << endl;
			setcolor( COLOR_NONE );
		}

		if( RC_NO != cm->connect_client() )
			return CM_ACK_NO;
		else
			return CM_ACK_OK;
	}

	// 현재 연결된 접속을 끊는다.
	if( strcmp( m_cmdBuffer, "disconnect" ) == 0 )
	{
		if( m_bDebug )
		{
			setcolor( COLOR_YELLOW );
			cout << "CM::disconnect" << endl;
			setcolor( COLOR_NONE );
		}
		cm->disconnect_client();
		return CM_ACK_OK;
	}

	// 원격지로 명령을 전송한다.
	// send 뒤에 오는 문자열을 원격지로 전송한다.
	// usage : send port 8888
	if( strncmp( m_cmdBuffer, "send ", strlen("send ") ) == 0  && 
		argc( m_cmdBuffer ) >= 2 )
	{
		if( strcmp( &m_cmdBuffer[5], "lock" ) == 0 )
		{
			cm->clientData.type = CM_LOCK;
		}
		else if( strcmp( &m_cmdBuffer[5], "unlock" ) == 0 )
		{
			cm->clientData.type = CM_UNLOCK;
		}
		else
		{
			cm->clientData.type = CM_TEST;
		}

		cm->clientData.len  = strlen( m_cmdBuffer ) - strlen("send ");
		strncpy( cm->clientData.data, &m_cmdBuffer[strlen("send ")], DATASIZ );
		return cm->sendCmd();
	}

	// lock을 건다.
	// 접속을 받아들이지 않고 동기화 시도를 하지 않는다.
	if( strcmp( m_cmdBuffer, "lock" ) == 0 )
	{
		lock();
		return CM_ACK_OK;
	}

	// lock을 해제한다.
	if( strcmp( m_cmdBuffer, "unlock" ) == 0 )
	{
		unlock();
		return CM_ACK_OK;
	}

	//---------------------------------------------------------------//
	// 미구현 기능들 
	//---------------------------------------------------------------//
	// 프로그램을 데몬으로 동작시킨다.
	if( strcmp( m_cmdBuffer, "demon" ) == 0 )
	{
		m_bDemonMode = true;
		if( fork() != 0 )
			exit(0);
		else
		{
			setsid();
			umask(0);
		}
	}

	// 상위 포트가 비어있으면 ListenPort를 상위로 이동시킨다.
	if( strcmp( m_cmdBuffer, "port refresh" ) == 0  ||
		strcmp( m_cmdBuffer, "pr" ) == 0 )
	{
		cm->portRefresh();
	}

	//---------------------------------------------------------------//
	// 테스트 기능들 
	//---------------------------------------------------------------//
	// 테스트 - 완전 테스트
	if( strcmp( m_cmdBuffer, "test" ) == 0 )
	{

	}

	// 현재 설정들을 출력한다.
	if( strcmp( m_cmdBuffer, "debug" ) == 0 ||
		strcmp( m_cmdBuffer, "d" ) == 0 ) 
	{
		system("clear");
		dispSettings();
	}

	// debugMode를 설정한다.
	if( strcmp( m_cmdBuffer, "debug on" ) == 0 )
	{
		debugON();
	}
	
	// debugMode를 해제한다.
	if( strcmp( m_cmdBuffer, "debug off" ) == 0 )
	{
		debugOFF();
	}

	// 레코드 락을 건다. 프로그램 중복 실행 방지용
	if( strcmp( m_cmdBuffer, "record lock" ) == 0 ||
	    strcmp( m_cmdBuffer, "rl" ) == 0 )
	{
		recordLock();
	}

	// 레코드 락을 해제한다.
	if( strcmp( m_cmdBuffer, "record unlock" ) == 0 ||
		strcmp( m_cmdBuffer, "rul" ) == 0 )
	{
		recordUnlock();
	}

	// 패스워드를 설정한다.
	if( strcmp( m_cmdBuffer, "set pass" ) == 0 )
	{
		setPassword();
	}

	// 문자열의 md5 해쉬코드를 리턴한다.
	// usage : md5str abc
	if( strncmp( m_cmdBuffer, "md5str ", strlen("md5str ") ) == 0  && 
		argc( m_cmdBuffer ) >= 2 )
	{
		char *str = strdup( &m_cmdBuffer[7] );
		cout << md5str( str ) << endl;
		free( str );
	}
	
	// 파일의 md5 해쉬코드를 리턴한다.
	// usage : md5file /home/sds/abc.txt
	if( strncmp( m_cmdBuffer, "md5file ", strlen("md5file ") ) == 0  && 
		argc( m_cmdBuffer ) == 2 )
	{
		cout << md5file( &m_cmdBuffer[8] ) << endl;
	}

	setcolor( COLOR_SKY );
	cout << "parsing::[" << m_cmdBuffer << "]" << endl;
	setcolor( COLOR_NONE ); 

	return CM_ACK_OK;
}

/*
 * 함수 : 레코드락
 * 설명 : 파일의 특정 부분에 레코드 락을 건다.
 */
bool ACHA::recordLock( void )
{
	m_fdRecordLock = open( recordLockPath, O_CREAT|O_TRUNC|O_RDWR, 0600 );

	if( flock( m_fdRecordLock, LOCK_EX|LOCK_NB ) < 0 )
	{
		cout << "프로그램이 이미 실행중입니다" << endl;
		return false;
	}
	return true;
}

/*
 * 함수 : 레코드락 해제
 * 설명 : 프로그램 중복실행 방지를 위한 함수
 */
bool ACHA::recordUnlock( void )
{
	// record lock 제거
	flock( m_fdRecordLock, LOCK_UN );

	close( m_fdRecordLock );
	m_fdRecordLock = -1;

	unlink( recordLockPath );

	return true;
}

/*
 * 함수 : 패스워드 입력받음
 */
void ACHA::setPassword( void )
{
    char buf1[SDATA], buf2[SDATA];

    bzero( buf1, sizeof(buf1) );
    bzero( buf2, sizeof(buf2) );

    while( 1 )
    {
        echoOFF();
        cout << "프로그램의 패스워드를 지정합니다." << endl;
        cout << "Password > ";
        fgets( buf1, SDATA, stdin );
        cout << endl << "Retype > ";
        fgets( buf2, SDATA, stdin );
        cout << endl;
        echoON();

        if( strlen( buf1 ) != strlen( buf2 ) ||
                strncmp( buf1, buf2, strlen( buf1 ) ) != 0 )
        {
            cout << "패스워드가 동일하지 않습니다." << endl;
            continue;
        }

        if( strlen( buf1 ) <= CM_PASS_LEN )
        {
            cout << "패스워드가 너무 짧습니다. 기본 " <<
                     CM_PASS_LEN << "글자 이상" << endl; 
            continue;
        }

        // '\n' 제거
        buf1[strlen(buf1)-1] = '\0';
        bzero( userpass, sizeof(userpass) );
        strncpy( userpass, buf1, sizeof(userpass) );
        break;
    }
    return;
}

/*
 * 함수 : 터미널 에코 OFF
 */
void ACHA::echoOFF( void )
{
    struct termios new_settings;

    tcgetattr( 0, &m_termios );

    new_settings = m_termios;
    new_settings.c_lflag &= (~ECHO);
    new_settings.c_lflag &= (~ICANON);

    tcsetattr( 0, TCSANOW, &new_settings );

    return ;
}

/*
 * 함수 : 터미널 에코 ON
 */
void ACHA::echoON( void )
{
    tcsetattr( 0, TCSANOW, &m_termios );

    return ;
}

/*
 * 함수 : 클래스 내부 변수들 확인
 */
void ACHA::dispSettings( void )
{
	CommunicationManager 	*cm	= (CommunicationManager *)m_pCommunicationManager;
	SyncManager				*sm	= (SyncManager *)m_pSyncManager;
	FileManager				*fm = (FileManager *)m_pFileManager;
//	UIModule				*ui = (UIModule *)m_pUIModule;

	setcolor( COLOR_SKY );
	cout << "[==================== ACHA Information ====================]\n";
	setcolor( COLOR_NONE );
	cout << "[1m";
	cout << " Lock / Debug    : " 	<< (m_bLock ? "true" : "false") << " / " 
									<< (m_bDebug ? "true" : "false") << endl;
	cout << " LoginID / PASS  : " << username << " / " << userpass << endl;
	cout << " RecordLockPath  : " << recordLockPath << " (" << m_fdRecordLock << ")" <<endl;
	cout << "[00m";

	sm->dispSettings();
	cm->dispSettings();
	fm->dispSettings();

	return;
}
/////////////////////////////////////////////////////////////////////////////////////////////

/*
 * 함수 : time_t 형 시간을 읽기 좋게 변경
 * 인자 : time_t 형 시간형식
 * 반환 : yymmdd-hh:mm:ss 형식의 문자열
 */
char *timeFormat( time_t now )
{
	struct tm *timeTmp = localtime( &now );
	static char buf[SDATA];
	bzero( buf, SDATA );

	sprintf( buf,
			 "%02d%02d%02d-%02d:%02d:%02d",
			 timeTmp->tm_year-100,
			 timeTmp->tm_mon+1,
			 timeTmp->tm_mday,
			 timeTmp->tm_hour,
			 timeTmp->tm_min,
			 timeTmp->tm_sec );

	return buf;
}

/*
 * 함수 : 타겟의 IP address를 받아 온다.
 * 설명 : 유효성도 체크한다.
 * 인자 : IP 혹은 Domain형식의 string
 * 반환 : IP 형식의 char형 포인터
 *        NULL : 유효하지 않은 입력
 */
char *getTargetIP( char *target )
{
	if( isIPformat( target ) )
	{
		return getTargetIP( target );
	}
	else
	{
		return NULL;
	}
}

/*
 * 함수 : 도메인 네임을 IP로 변환
 * 인자 : 도메인 네임형 스트링
 * 반환 : IP 형식의 char형 포인터
 */
char *getIPfromDomain( char *domain )
{
	struct hostent *he;
	static char buf[SDATA];

	bzero( buf, SDATA );
	he = gethostbyname( domain );
	strncpy( buf, inet_ntoa( *(struct in_addr*)he->h_addr), SDATA );

	return buf;
}

/*
 * 함수 : 입력받은 스트링이 IP형식인지 확인
 * 반환 : IP형식 - true    else   false
 */
bool isIPformat( char *str )
{
	int a, b, c, d;
	sscanf( str, "%d.%d.%d.%d", &a, &b, &c, &d );
	if( a >  0  && a < 256  &&
		b >= 0  && b < 256  &&
		c >= 0  && d < 256  &&
		d >  0  && d < 256 )
	{
		return true;
	}

	return false;
}

/*
 * 함수 : 원격지 주소를 생성한다.
 */
char *gen_rpath( char *lp, char *rp, char *fullpath )
{
	static char buf[DATASIZ];
	bzero( buf, DATASIZ );

	char *filepath = &fullpath[strlen(lp)];
	if( rp[strlen(rp)-1] == '/' && filepath[0] == '/' )
	{
		sprintf( buf, "%s%s", rp, &filepath[1] );

	}
	else if( ( rp[strlen(rp)-1] != '/' && filepath[0] == '/' ) || 
			 ( rp[strlen(rp)-1] == '/' && filepath[0] != '/' ) )
	{
		sprintf( buf, "%s%s", rp, filepath );
	}
	else if( rp[strlen(rp)-1] != '/' && filepath[0] != '/' )
	{
		sprintf( buf, "%s/%s", rp, filepath );
	}

	return buf;
}

/*
 * 함수 : 인자 갯수를 리턴한다.
 */
int argc( char *str )
{
	int count;
	char *tok;
	char buf[BUFSIZ] = "";
	strncpy( buf, str, BUFSIZ );

	tok = strtok( buf, DELIM );
	for( count = 0; tok != NULL; count++ )
	{
		tok = strtok( NULL, DELIM );
	}

	return count;
}

/*
 * 함수 : 텍스트 출력 컬러 설정
 */
void setcolor( int color )
{
	cout << "[00m";

	switch( color )
	{
		case COLOR_RED :
			cout << "[1;31m";
			break;
		case COLOR_GREEN:
			cout << "[1;32m";
			break;
		case COLOR_YELLOW :
			cout << "[1;33m";
			break;
		case COLOR_BLUE :
			cout << "[1;34m";
			break;
		case COLOR_PINK :
			cout << "[1;35m";
			break;
		case COLOR_SKY :
			cout << "[1;36m";
			break;
		case COLOR_WHITE :
			cout << "[1;37m";
			break;
		case COLOR_DARKRED :
			cout << "[31m";
			break;
		case COLOR_DARKGREEN :
			cout << "[32m";
			break;
		case COLOR_BROWN :
			cout << "[33m";
			break;
		case COLOR_DARKBLUE :
			cout << "[34m";
			break;
		case COLOR_PUPLE :
			cout << "[35m";
			break;
		case COLOR_DARKSKY :
			cout << "[36m";
			break;
		case COLOR_GRAY :
			cout << "[37m";
			break;
		default :
			cout << "[00m";
			break;
	}

	return;
}

/*
 * 함수 : string 의 MD5 해쉬코드를 리턴
 */
char *md5str( char *str )
{
	unsigned char *originStr = (unsigned char *)strdup(str);
	unsigned char md5[16];
	unsigned long len;
    static char md5msg[40];
	int i;

	memset(md5, 0, 16);

	len = strlen(str);
	MD5(originStr, len, md5);

	bzero( md5msg, 40 );
	for( i = 0; i < 16; i ++ )
	{
        sprintf(md5msg+(i*2), "%02X", md5[i]);
	}

	return md5msg;
}

/*
 * 함수 : file 의 MD5 해쉬코드를 리턴
 */
char *md5file( char *path )
{
    MD5_CTX lctx;
    unsigned char digest[16];
    static char md5msg[40];
    unsigned int i;
    int fd;
    struct stat status;
    char *data;

    MD5_Init(&lctx);

    fd = open(path, O_RDONLY);
    if (fd < 0)
    {
		cout << "MD5 :: Can't Open File" << endl;
		return NULL;
    }
    fstat(fd, &status);

    data = (char *)malloc(status.st_size);
    read (fd, data, status.st_size);

    MD5_Update(&lctx, data, status.st_size);
    MD5_Final(digest, &lctx);

	bzero( md5msg, 40 );
    for (i = 0; i < sizeof(digest); ++i)
    {
        sprintf(md5msg+(i*2), "%02X", digest[i]);
    }

    free( data );
	close( fd );

    return md5msg;
}

// 파일의 퍼미션을 읽어서 리턴한다
mode_t getPermission( char *path )
{
	struct stat status;
	mode_t filePermission;

	stat( path, &status );
	filePermission 	= status.st_mode;

	return filePermission;
}
